%feature("docstring") OT::HSICEstimatorGlobalSensitivity
"Implement a HSIC estimator for global analysis.

Parameters
----------
covarianceModelCollection : list of :class:`~openturns.CovarianceModel`
    List of all covariance kernels. The :math:`d` first kernels are linked to the input 
    and the last one is for the output.

X : 2-d sequence of float
    The input sample used for the HSIC analysis of dimension :math:`d`.

Y : 2-d sequence of float
    The output sample used for the HSIC analysis of dimension 1.

estimatorType : :class:`~openturns.HSICStat`
    An estimator for internal computations.

See also
--------
:class:`~openturns.HSICEstimatorConditionalSensitivity`, :class:`~openturns.HSICEstimatorTargetSensitivity`

Examples
--------
>>> import openturns as ot
>>> from math import pi
>>> ot.RandomGenerator.SetSeed(0)

Generate input and output samples.

>>> # 3d input distribution with an independent copula
>>> distX = ot.JointDistribution([ot.Uniform(-pi, pi)] * 3)
>>> X = distX.getSample(100) # get a sample
>>>
>>> # Apply the Ishigami model.
>>> inputs = ['X1', 'X2', 'X3']
>>> formula = ['sin(X1) + 5.0 * (sin(X2))^2 + 0.1 * X3^4 * sin(X1)']
>>> modelIshigami = ot.SymbolicFunction(inputs, formula)
>>> Y = modelIshigami(X) # Y = modelIshigami(X)

Define covariance kernels for the model inputs.
Put them in a list.

>>> covarianceModelCollection = []
>>> for i in range(3):
...     Xi = X.getMarginal(i)
...     Cov = ot.SquaredExponential(1)
...     Cov.setScale(Xi.computeStandardDeviation())
...     covarianceModelCollection.append(Cov)

Append the list with the covariance kernel for the model output.

>>> covarianceModelCollection.append(ot.SquaredExponential(Y.computeStandardDeviation()))

Choose the statistic that will be used to estimate the HSIC indices.

>>> estimatorType = ot.HSICUStat() # could be HSICVStat instead

Build and use the HSIC estimator for global sensitivity analysis.

>>> hsic = ot.HSICEstimatorGlobalSensitivity(covarianceModelCollection, X, Y, estimatorType)
>>> print(hsic.getR2HSICIndices())
[0.298073,0.00344498,0.0772657]"

// ---------------------------------------------------------------------
